%MACRO BRR_EFFECT_nPV(INFILE=,
				     REPLI_ROOT=,	
				     BYVAR=,
				     PV_ROOT=,
					 NB_PV=,
				     EFFECT=,
				     OUTFILE=);

/* 

MEANING OF THE MACRO ARGUMENTS

INFILE		= 	INPUT DATA FILE.
REPLI_ROOT	= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
				FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR		= 	BREAKDOWN VARIABLES
PV_ROOT		= 	ROOT OF THE PLAUSIBLE VALUES VARIABLES NAMES
NB_PV		=	NUMBER OF PLAUSIBLE VALUES
EFFECT		=	THIS ARGUMENT HAS THREE COMPONENTS: THE VARIABLE NAME AND THE TWO CATEGORIES ON WHICH THE EFFECT
				SIZE WILL BE COMPUTED. THE EFFECT SIZE WILL BE THE SECOND COMPONENT MINUS THE THIRD COMPONENT
OUTFILE		= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 

%LET GROUP=%SCAN(&EFFECT,1);
%LET CAT1=%SCAN(&EFFECT,2);
%LET CAT2=%SCAN(&EFFECT,3);

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1;
RUN;

DATA BRRDATA;
	SET &INFILE;
	IF (&GROUP=&CAT1) THEN VAR_GROUP=1;
	IF (&GROUP=&CAT2) THEN VAR_GROUP=2;
	IF (VAR_GROUP IN (1,2));
	KEEP &REPLI_ROOT.0-&REPLI_ROOT.80 &PV_ROOT.1-&PV_ROOT&NB_PV &BYVAR  VAR_GROUP;
RUN;

PROC SORT DATA=BRRDATA;
  BY &BYVAR VAR_GROUP;
RUN;

%DO ITER_1 = 0 %TO 80;
 	PROC MEANS DATA=BRRDATA VARDEF=WGT NOPRINT;
		VAR &PV_ROOT.1 - &PV_ROOT&NB_PV  ;
		BY &BYVAR VAR_GROUP;
		WEIGHT &REPLI_ROOT&ITER_1;
		OUTPUT OUT=MEAN_TEMP MEAN=MOY1-MOY&NB_PV VAR=VARI1-VARI&NB_PV;
	RUN;

	DATA MEAN_TEMP;
		SET MEAN_TEMP;
		L=&ITER_1;
	RUN;

	PROC APPEND BASE = BRR_TEMP1 DATA=MEAN_TEMP;
 	RUN;

%END;

PROC SORT DATA=BRR_TEMP1;
	BY &BYVAR L VAR_GROUP;
RUN;
DATA BRR_TEMP1B; SET BRR_TEMP1; 
	ARRAY VECTR_1 (&NB_PV) MOY1-MOY&NB_PV;
	ARRAY VECTR_2 (&NB_PV) V1-V&NB_PV;
	DO BOUCLE_1=1 TO &NB_PV; 
		VECTR_2(BOUCLE_1)=VECTR_1(BOUCLE_1); 
	END; 
	DROP MOY1-MOY&NB_PV; 
RUN;
PROC TRANSPOSE DATA=BRR_TEMP1B OUT=BRR_TEMP2 PREFIX=M;
	VAR V1-V&NB_PV	;
	BY &BYVAR L;
	ID VAR_GROUP;
RUN;
DATA BRR_TEMP1C; SET BRR_TEMP1; 
	ARRAY VECTR_3 (&NB_PV) VARI1-VARI&NB_PV;
	ARRAY VECTR_4 (&NB_PV) V1-V&NB_PV;
	DO BOUCLE_2=1 TO &NB_PV; 
		VECTR_4(BOUCLE_2)=VECTR_3(BOUCLE_2); 
	END; 
	DROP VARI1-VARI&NB_PV; 
RUN;
PROC TRANSPOSE DATA=BRR_TEMP1C OUT=BRR_TEMP3 PREFIX=V;
	VAR V1-V&NB_PV	;
	BY &BYVAR L;
	ID VAR_GROUP;
RUN;
DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY 	&BYVAR L _NAME_;
	PV=(M1-M2)/(((V1+V2)/2)**0.5);
	KEEP  &BYVAR L PV _NAME_;
RUN;
PROC TRANSPOSE DATA=BRR_TEMP4 OUT=BRR_TEMP5(DROP=_NAME_) PREFIX=P;
	VAR PV;
	BY &BYVAR L;
	ID _NAME_;
RUN;

DATA BRR_TEMP6(DROP=STAT FIN1-FIN&NB_PV MESVAR) BRR_TEMP7(KEEP=&BYVAR STAT FIN1-FIN&NB_PV MESVAR FLAG_ST);
 	SET BRR_TEMP5;
    IF L > 0 THEN OUTPUT BRR_TEMP6;     
    ELSE DO;
       	STAT =SUM (OF PV1-PV&NB_PV)/&NB_PV;
		MESVAR=VAR (OF PV1-PV&NB_PV);
		ARRAY VECTR_5 (&NB_PV) PV1-PV&NB_PV; 
		ARRAY VECTR_6 (&NB_PV) FIN1-FIN&NB_PV; 
		DO BOUCLE_3=1 TO &NB_PV; 
	   		VECTR_6(BOUCLE_3)=VECTR_5(BOUCLE_3);
	   	END;
		ARRAY VECTR_7 (&NB_PV) PV1-PV&NB_PV; 
		DO BOUCLE_4=1 TO &NB_PV; 
			IF VECTR_7(BOUCLE_4) IN (.,.M,.N,.I) THEN FLAG_ST=+1; 
		END;
		OUTPUT BRR_TEMP7;
   	END;
RUN;

PROC SORT DATA=BRR_TEMP6;
 	BY &BYVAR;
RUN;

PROC SORT DATA=BRR_TEMP7;
 	BY &BYVAR;
RUN;

DATA BRR_TEMP8;
	MERGE BRR_TEMP6 BRR_TEMP7;
	BY &BYVAR;
	ARRAY VECTR_8 (&NB_PV) PV1-PV&NB_PV;
	ARRAY VECTR_9 (&NB_PV) FIN1-FIN&NB_PV;
	ARRAY VECTR_10 (&NB_PV) VAR1-VAR&NB_PV;
	DO BOUCLE_5=1 TO &NB_PV;
	VECTR_10(BOUCLE_5)=(1/20)*((VECTR_8(BOUCLE_5)-VECTR_9(BOUCLE_5))**2);
	END;
RUN;

PROC UNIVARIATE DATA=BRR_TEMP8 NOPRINT;
	VAR VAR1-VAR&NB_PV;
	BY &BYVAR;
	OUTPUT OUT=BRR_TEMP9 SUM=SS1-SS&NB_PV;
RUN;

DATA &OUTFILE;
	MERGE BRR_TEMP7 BRR_TEMP9;
	BY &BYVAR;
	SAMP=SUM (OF SS1-SS&NB_PV)/&NB_PV;
	FACT_IMP=1+(1/&NB_PV);
	FINVAR=(SAMP+(FACT_IMP*MESVAR));
	SESTAT=(FINVAR)**0.5;
	IF FLAG_ST>=1 THEN DO; STAT=.; SESTAT=.; END; 
	FORMAT STAT F10.2;
	FORMAT SESTAT F10.2;
	KEEP &BYVAR STAT SESTAT  ;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRRDATA MEAN_TEMP BRR_TEMP1: BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 BRR_TEMP6 BRR_TEMP7 BRR_TEMP8 BRR_TEMP9;
RUN;

OPTIONS NOTES;

%MEND;
