%MACRO BRR_PROCMEAN_DIF_nPV(INFILE =,
                           REPLI_ROOT =,
                           BYVAR =,
    	                   PV_ROOT =,
 						   NB_PV=,
                           COMPARE =,
	                       CATEGORY =,
                           STAT =,
                           OUTFILE =);

/*

MEANING OF THE MACRO ARGUMENTS

INFILE		= 	INPUT DATA FILE.
REPLI_ROOT	= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
				FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR		= 	BREAKDOWN VARIABLES
PV_ROOT		= 	ROOT OF THE PLAUSIBLE VALUES VARIABLES NAMES
NB_PV		=	NUMBER OF PLAUSIBLE VALUES
COMPARE		=	BREAKDOWN VARIABLE NAME FOR WICH CATEGORY CONTRASTS ARE REQUESTED
CATEGORY	=	LIST OF THE "COMPARE" VARIABLE CATEGORIES FOR WHICH A CONTRAST IS REQUESTED
STAT		= 	REQUESTED STATISTIQUE
				SUMWGT		=	SUM OF THE WEIGHT
				MEAN		= 	MEAN
				VAR			=	VARIANCE
				STD			=	STANDARD DEVIATION
				CV			=	COEFFICIENT OF VARIATION
				SKEWNESS	=	SKEWNESS
				KURTOSIS	=	KURTOSIS
				MEDIAN		=	MEDIAN
				Q1			=	FIRST QUARTILE
				Q3			=	THIRD QUARTILE
				QRANGE		=	RANGE BETWEEN Q1 AND Q3
				PX			=	PERCENTILE, WITH X BETWEEN 1 AND 99
OUTFILE		= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0 BRR_TEMP1 ;
RUN;

PROC SORT DATA=&INFILE OUT=BRRDATA(KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &COMPARE &PV_ROOT.1-&PV_ROOT&NB_PV);
	BY &BYVAR &COMPARE;
RUN;

%DO ITER_1 = 0 %TO 80;

	PROC MEANS DATA=BRRDATA VARDEF=WGT NOPRINT;
		VAR &PV_ROOT.1 - &PV_ROOT&NB_PV ;
		BY &BYVAR &COMPARE;
		WEIGHT &REPLI_ROOT&ITER_1;
		OUTPUT OUT=MEAN_TEMP &STAT=PV1 - PV&NB_PV;
 	RUN;

	DATA MEAN_TEMP;
		SET MEAN_TEMP;
		L=&ITER_1;
 	RUN;

	PROC APPEND BASE = BRR_TEMP0 DATA=MEAN_TEMP;
	RUN;

%END;

%LET DEBUT=1;
%LET SUIVANT=2;
%LET COMPTE=1;
%LET I=1;

%DO %WHILE(%LENGTH(%SCAN(&CATEGORY,&I)));
	%LET I=%EVAL(&I+1);
%END;

%LET NBCAT=%EVAL(&I-1);
%LET NBDIF=%EVAL((&NBCAT*(&NBCAT-1))/2);
%LET ORDRE=1;

%DO ITER_2=1 %TO &NBDIF;
	%DO ITER_3=&DEBUT %TO &NBCAT-1;
		%LET CAT1=%SCAN(&CATEGORY,&DEBUT);
		%LET CAT2=%SCAN(&CATEGORY,&SUIVANT);

		DATA BRR_DIF1;
			 SET BRR_TEMP0;
			 ARRAY VECTR_1 (&NB_PV) PV1-PV&NB_PV; 
			 ARRAY VECTR_2 (&NB_PV) M1PV1-M1PV&NB_PV; 
			 DO BOUCLE_1=1 TO &NB_PV; 
		   		VECTR_2(BOUCLE_1)=VECTR_1(BOUCLE_1);
			 END;
			 LENGTH CONTRAST $5;
			 CONTRAST="&CAT1.-&CAT2";
			 IF (&COMPARE=&CAT1);
			 KEEP &BYVAR L CONTRAST M1PV1-M1PV&NB_PV &COMPARE;
		RUN;

		DATA BRR_DIF2 ;
		 	 SET BRR_TEMP0;
			 ARRAY VECTR_3 (&NB_PV) PV1-PV&NB_PV; 
			 ARRAY VECTR_4 (&NB_PV) M2PV1-M2PV&NB_PV; 
			 DO BOUCLE_2=1 TO &NB_PV; 
		   		VECTR_4(BOUCLE_2)=VECTR_3(BOUCLE_2);
			 END;
		     IF (&COMPARE=&CAT2);
			 KEEP &BYVAR L M2PV1-M2PV&NB_PV &COMPARE;
		RUN;

		PROC SORT DATA=BRR_DIF1;
			BY &BYVAR L;
		RUN;

		PROC SORT DATA=BRR_DIF2;
			BY &BYVAR L;
		RUN;

		DATA BRR_TEMP_DIF;
			MERGE BRR_DIF1 BRR_DIF2;
			BY &BYVAR L;
			IF (M1PV1 EQ . OR M2PV1 EQ .) THEN DELETE;
			ARRAY VECTR_5 (&NB_PV) M1PV1-M1PV&NB_PV; 
			ARRAY VECTR_6 (&NB_PV) M2PV1-M2PV&NB_PV; 
			ARRAY VECTR_7 (&NB_PV) PV1-PV&NB_PV;
			DO BOUCLE_3=1 TO &NB_PV; 
		   		VECTR_7(BOUCLE_3)=VECTR_5(BOUCLE_3)-VECTR_6(BOUCLE_3);
			END;
			ORDRE=&ORDRE;
			KEEP &BYVAR CONTRAST L PV1-PV&NB_PV ORDRE;
		RUN;

 		PROC APPEND BASE = BRR_TEMP1 DATA=BRR_TEMP_DIF;
		RUN;

		%LET SUIVANT=%EVAL(&SUIVANT+1);
		%LET COMPTE=%EVAL(&COMPTE+1);
		%LET ORDRE=%EVAL(&ORDRE+1);

	%END;

%LET DEBUT=%EVAL(&DEBUT+1);
%LET SUIVANT=%EVAL(&DEBUT+1);

%END;

DATA BRR_TEMP2(DROP=STAT FIN1-FIN&NB_PV MESVAR) BRR_TEMP3(KEEP=&BYVAR CONTRAST ORDRE STAT FIN1-FIN&NB_PV MESVAR FLAG_ST);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
       	STAT =SUM (OF PV1-PV&NB_PV)/&NB_PV;
		MESVAR=VAR (OF PV1-PV&NB_PV);
		ARRAY VECTR_8 (&NB_PV) PV1-PV&NB_PV; 
		ARRAY VECTR_9 (&NB_PV) FIN1-FIN&NB_PV; 
		DO BOUCLE_4=1 TO &NB_PV; 
	   		VECTR_9(BOUCLE_4)=VECTR_8(BOUCLE_4);
	   	END;
		ARRAY VECTR_10 (&NB_PV) PV1-PV&NB_PV; 
		DO BOUCLE_5=1 TO &NB_PV; 
			IF VECTR_10(BOUCLE_5) IN (.,.M,.N,.I) THEN FLAG_ST=+1; 
		END;
		OUTPUT BRR_TEMP3;
   END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR CONTRAST;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR CONTRAST;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR CONTRAST;
	ARRAY VECTR_11 (&NB_PV) PV1-PV&NB_PV;
	ARRAY VECTR_12 (&NB_PV) FIN1-FIN&NB_PV;
	ARRAY VECTR_13 (&NB_PV) VAR1-VAR&NB_PV;
	DO BOUCLE_6=1 TO &NB_PV;
	VECTR_13(BOUCLE_6)=(1/20)*((VECTR_11(BOUCLE_6)-VECTR_12(BOUCLE_6))**2);
	END;
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VAR1-VAR&NB_PV;
	BY &BYVAR CONTRAST;
	OUTPUT OUT=BRR_TEMP5 SUM=SS1-SS&NB_PV;
RUN;

DATA BRR_TEMP6;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR CONTRAST;
	SAMP=SUM (OF SS1-SS&NB_PV)/&NB_PV;
	FACT_IMP=1+(1/&NB_PV);
	FINVAR=(SAMP+(FACT_IMP*MESVAR));
	SESTAT=(FINVAR)**0.5;
	IF FLAG_ST>=1 THEN DO; STAT=.; SESTAT=.; END; 
	KEEP &BYVAR CONTRAST ORDRE STAT SESTAT;
RUN;

PROC SORT DATA=BRR_TEMP6 OUT=&OUTFILE (DROP=ORDRE);
	BY &BYVAR ORDRE;
RUN;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0 BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 BRR_TEMP6
	MEAN_TEMP BRRDATA BRR_DIF1 BRR_DIF2 BRR_TEMP_DIF;
RUN;

OPTIONS NOTES;

%MEND BRR_PROCMEAN_DIF_nPV;

