%macro BRR_FREQ_nPV(INFILE=,
				   REPLI_ROOT=,
				   BYVAR=,
				   PV_ROOT=,
				   NB_PV=,
				   LIMIT=,
				   LIMIT_CRITERIA=,
				   ID_SCHOOL=,
				   OUTFILE=);

/* 

MEANING OF THE MACRO ARGUMENTS

INFILE			= 	INPUT DATA FILE.
REPLI_ROOT		= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
					FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR			= 	BREAKDOWN VARIABLES
PV_ROOT			= 	ROOT OF THE PROFICIENCY LEVEL VARIABLES NAMES
NB_PV			=	NUMBER OF PLAUSIBLE VALUES
LIMIT          	=   FLAGGING  YES OR NO
LIMIT_CRITERIA 	=   1) NUMBER OF STUDENTS 2) NUMBER OF SCHOOLS 3) PERCENTAGE OF STUDENTS AND 4) NUMBER OF VARIABLES FROM 
					THE BYVAR ARGUMENT FOR DEFINING THE POPULATION OF REFERENCE 
ID_SCHOOL	   	=	VARIABLE NAME FOR THE SCHOOL IDENTIFICATION
OUTFILE			= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0;
RUN;

%DO ITER_1 = 1 %TO &NB_PV;

	PROC SORT DATA=&INFILE
		OUT=BRRDATA(KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &PV_ROOT.1-&PV_ROOT&NB_PV &ID_SCHOOL);
		BY &BYVAR &PV_ROOT&ITER_1;
	RUN;

	%DO ITER_2 = 0 %TO 80;

		PROC FREQ DATA=BRRDATA NOPRINT;
			TABLE &PV_ROOT&ITER_1 /OUT=FREQ_TEMP ;
			BY &BYVAR;
			WEIGHT &REPLI_ROOT&ITER_2;
		RUN;

		DATA FREQ_TEMP;
			SET FREQ_TEMP;
			K=&ITER_1;
			L=&ITER_2;
			&PV_ROOT=&PV_ROOT&ITER_1;
			KEEP &BYVAR K L PERCENT &PV_ROOT;
		RUN;

		PROC APPEND BASE = BRR_TEMP0 DATA=FREQ_TEMP;
		RUN;

	%END;

%END;

PROC SORT DATA=BRR_TEMP0;
	BY &BYVAR L &PV_ROOT;
RUN;

PROC TRANSPOSE DATA=BRR_TEMP0 OUT=BRR_TEMP1 PREFIX=PV;
	BY &BYVAR L &PV_ROOT;
	VAR PERCENT;
RUN; 

DATA BRR_TEMP1;
	SET BRR_TEMP1;
	DROP _NAME_ _LABEL_;
RUN;

DATA BRR_TEMP2(DROP=STAT FIN1-FIN&NB_PV MESVAR) BRR_TEMP3(KEEP=&BYVAR &PV_ROOT STAT FIN1-FIN&NB_PV MESVAR FLAG_ST);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
       	STAT =SUM (OF PV1-PV&NB_PV)/&NB_PV;
		MESVAR=VAR (OF PV1-PV&NB_PV);
		ARRAY VECTR_1 (&NB_PV) PV1-PV&NB_PV; 
		ARRAY VECTR_2 (&NB_PV) FIN1-FIN&NB_PV; 
		DO BOUCLE_1=1 TO &NB_PV; 
	   		VECTR_2(BOUCLE_1)=VECTR_1(BOUCLE_1);
	   	END;
		ARRAY VECTR_3 (&NB_PV) PV1-PV&NB_PV; 
		DO BOUCLE_2=1 TO &NB_PV; 
			IF VECTR_3(BOUCLE_2) IN (.,.M,.N,.I) THEN FLAG_ST=+1; 
		END;

		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR &PV_ROOT;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR &PV_ROOT;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR &PV_ROOT;
	ARRAY VECTR_4 (&NB_PV) PV1-PV&NB_PV;
	ARRAY VECTR_5 (&NB_PV) FIN1-FIN&NB_PV;
	ARRAY VECTR_6 (&NB_PV) VAR1-VAR&NB_PV;
	DO BOUCLE_3=1 TO &NB_PV;
	VECTR_6(BOUCLE_3)=(1/20)*((VECTR_4(BOUCLE_3)-VECTR_5(BOUCLE_3))**2);
	END;
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VAR1-VAR&NB_PV;
	BY &BYVAR &PV_ROOT;
	OUTPUT OUT=BRR_TEMP5 SUM=SS1-SS&NB_PV;
RUN;

DATA BRR_TEMP6;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR &PV_ROOT;
	SAMP=SUM (OF SS1-SS&NB_PV)/&NB_PV;
	FACT_IMP=1+(1/&NB_PV);
	FINVAR=(SAMP+(FACT_IMP*MESVAR));
	SESTAT=(FINVAR)**0.5;
	IF FLAG_ST>=1 THEN DO; STAT=.; SESTAT=.; END; 
	KEEP &BYVAR &PV_ROOT STAT SESTAT;
RUN;

%IF (%UPCASE(&LIMIT)=NO) %THEN %DO;

	DATA &OUTFILE;
		SET BRR_TEMP6;
	RUN;

%END;

%ELSE %DO;
	%DO ITER_3=1 %TO &NB_PV;
	PROC FREQ DATA=BRRDATA NOPRINT;
		TABLE &PV_ROOT&ITER_3/OUT=BRR_TEMP7;
		BY &BYVAR;
	RUN;
	DATA BRR_TEMP8;
		SET BRR_TEMP7;
		COUNT&ITER_3=COUNT;
		&PV_ROOT=&PV_ROOT&ITER_3;
		KEEP &BYVAR &PV_ROOT COUNT&ITER_3;
	RUN;
	%IF (&ITER_3=1) %THEN %DO;
		DATA BRR_TEMP9;
			SET BRR_TEMP8;
		RUN;
	%END;
	%ELSE %DO;
		DATA BRR_TEMP9;
			MERGE BRR_TEMP9 BRR_TEMP8;
			BY &BYVAR &PV_ROOT;
		RUN;
	%END;
	%END;
	%LET FLAG_STUD=%SCAN(&LIMIT_CRITERIA,1);
	DATA BRR_TEMP9;
		SET BRR_TEMP9;
		COUNT=SUM (OF COUNT1 - COUNT&NB_PV)/&NB_PV;

		ARRAY VECTR_X1 (&NB_PV) COUNT1 - COUNT&NB_PV; 
		DO BOUCLE_X1=1 TO &NB_PV; 
			IF VECTR_X1(BOUCLE_X1) IN (.,.M,.N,.I) THEN FLAG_MI=+1; 
		END;
		IF FLAG_MI>=1 THEN COUNT=.; 		

		IF (COUNT < &FLAG_STUD) THEN FLAG_STUD=1;
		ELSE FLAG_STUD=0;
		KEEP &BYVAR &PV_ROOT FLAG_STUD;
	RUN;

	PROC SORT DATA=BRRDATA;
		BY &BYVAR  &ID_SCHOOL;
	RUN;

	%DO ITER_4=1 %TO &NB_PV;
	PROC FREQ DATA=BRRDATA NOPRINT;
		TABLE &PV_ROOT&ITER_4 /OUT=BRR_TEMP10;
		BY &BYVAR  &ID_SCHOOL;
	RUN;	
	PROC FREQ DATA=BRR_TEMP10 NOPRINT;
		TABLE &PV_ROOT&ITER_4  /OUT=BRR_TEMP11;
		BY &BYVAR;
	RUN;
	DATA BRR_TEMP12;
		SET BRR_TEMP11;
		COUNT&ITER_4=COUNT;
		&PV_ROOT=&PV_ROOT&ITER_4;
		KEEP &BYVAR &PV_ROOT COUNT&ITER_4;
	RUN;
	%IF (&ITER_4=1) %THEN %DO;
		DATA BRR_TEMP13;
			SET BRR_TEMP12;
		RUN;
	%END;
	%ELSE %DO;
		DATA BRR_TEMP13;
			MERGE BRR_TEMP13 BRR_TEMP12;
			BY &BYVAR &PV_ROOT;
		RUN;
	%END;
	%END;
	%LET FLAG_SCH=%SCAN(&LIMIT_CRITERIA,2);
	DATA BRR_TEMP13;
		SET BRR_TEMP13;
		COUNT=SUM (OF COUNT1 - COUNT&NB_PV)/&NB_PV;

		ARRAY VECTR_X2 (&NB_PV) COUNT1 - COUNT&NB_PV; 
		DO BOUCLE_X2=1 TO &NB_PV; 
			IF VECTR_X2(BOUCLE_X2) IN (.,.M,.N,.I) THEN FLAG_MI=+1; 
		END;
		IF FLAG_MI>=1 THEN COUNT=.; 	

		IF (COUNT < &FLAG_SCH) THEN FLAG_SCH=1;
		ELSE FLAG_SCH=0;
		KEEP &BYVAR &PV_ROOT FLAG_SCH;
	RUN;

	%DO ITER_5=1 %TO &NB_PV;
	PROC FREQ DATA=BRRDATA NOPRINT;
		TABLE &PV_ROOT&ITER_5/OUT=BRR_TEMP14;
		BY &BYVAR;
		WEIGHT &REPLI_ROOT.0;
	RUN;
	DATA BRR_TEMP15;
		SET BRR_TEMP14;
		COUNT&ITER_5=COUNT;
		&PV_ROOT=&PV_ROOT&ITER_5;
		KEEP &BYVAR &PV_ROOT COUNT&ITER_5;
	RUN;
	%IF (&ITER_5=1) %THEN %DO;
		DATA BRR_TEMP16;
			SET BRR_TEMP15;
		RUN;
	%END;
	%ELSE %DO;
		DATA BRR_TEMP16;
			MERGE BRR_TEMP16 BRR_TEMP15;
			BY &BYVAR &PV_ROOT;
		RUN;
	%END;
	%END;

	%LET K=1;
	%LET POPREF=;
	%LET NBVAR=%SCAN(&LIMIT_CRITERIA,4);
	%DO %WHILE(&K <= &NBVAR);
		%LET POPREF_ADD=%SCAN(&BYVAR,&K);
		%LET POPREF=&POPREF &POPREF_ADD;
		%LET K=%EVAL(&K+1);
	%END;

	PROC MEANS DATA=BRR_TEMP16 NOPRINT;
		VAR COUNT1-COUNT&NB_PV ;
		BY &POPREF;
		OUTPUT OUT=BRR_TEMP17 SUM=SOMWGT1-SOMWGT&NB_PV;
	RUN;
	%LET FLAG_PCT=%SCAN(&LIMIT_CRITERIA,3);
	DATA BRR_TEMP18;
		MERGE BRR_TEMP16 BRR_TEMP17;
		BY &POPREF;
		ARRAY VECTR_7 (&NB_PV) PCT1-PCT&NB_PV;
		ARRAY VECTR_8 (&NB_PV) COUNT1-COUNT&NB_PV;
		ARRAY VECTR_9 (&NB_PV) SOMWGT1-SOMWGT&NB_PV;
		DO BOUCLE_4=1 TO &NB_PV;
			VECTR_7(BOUCLE_4)=(VECTR_8(BOUCLE_4)/VECTR_9(BOUCLE_4))*100;
		END;
		PCT=SUM (OF PCT1-PCT&NB_PV)/&NB_PV;

		ARRAY VECTR_X3 (&NB_PV) COUNT1 - COUNT&NB_PV; 
		DO BOUCLE_X3=1 TO &NB_PV; 
			IF VECTR_X3(BOUCLE_X3) IN (.,.M,.N,.I) THEN FLAG_MI=+1; 
		END;
		IF FLAG_MI>=1 THEN PCT=.; 

		IF (PCT < &FLAG_PCT) THEN FLAG_PCT=1;
		ELSE FLAG_PCT=0;
		KEEP &BYVAR &PV_ROOT FLAG_PCT;
	RUN;
	
	DATA &OUTFILE;
		MERGE BRR_TEMP6 BRR_TEMP9 BRR_TEMP13 BRR_TEMP18;
		BY &BYVAR &PV_ROOT;
	RUN;
		
	PROC DATASETS LIBRARY=WORK NOLIST;
		DELETE BRR_TEMP7 BRR_TEMP8 BRR_TEMP9 BRR_TEMP10 BRR_TEMP11 BRR_TEMP12 BRR_TEMP13
				BRR_TEMP14 BRR_TEMP15 BRR_TEMP16 BRR_TEMP17 BRR_TEMP18;
	RUN;

%END;


PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0 BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 FREQ_TEMP BRR_TEMP6 MEAN_TEMP BRRDATA;
RUN;

OPTIONS NOTES;

%MEND BRR_FREQ_nPV;
