%macro BRR_REG_nPV(INFILE=,
			      REPLI_ROOT=,
			      EXPLICA=,
			      BYVAR=,
			      PV_ROOT=,
				  NB_PV=,
			      LIMIT=,
			      LIMIT_CRITERIA=,
			      ID_SCHOOL=,
			      OUTFILE=);

/*

MEANING OF THE MACRO ARGUMENTS

INFILE		= 	INPUT DATA FILE.
REPLI_ROOT	= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
				FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
EXPLICA		=	LIST OF INDEPENDENT VARIABLES
BYVAR		= 	BREAKDOWN VARIABLES
PV_ROOT		= 	ROOT OF THE PLAUSIBLE VALUES VARIABLES NAMES
NB_PV		=	NUMBER OF PLAUSIBLE VALUES
LIMIT          =    FLAGGING  YES OR NO
LIMIT_CRITERIA =    1) NUMBER OF STUDENTS 2) NUMBER OF SCHOOLS 3) PERCENTAGE OF STUDENTS AND 4) NUMBER OF VARIABLES FROM 
					THE BYVAR ARGUMENT FOR DEFINING THE POPULATION OF REFERENCE 
ID_SCHOOL	   =	VARIABLE NAME FOR THE SCHOOL IDENTIFICATION
OUTFILE		=	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/

OPTIONS NONOTES;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1;
RUN;

PROC SORT DATA=&INFILE OUT=BRRDATA (KEEP=&REPLI_ROOT.0-&REPLI_ROOT.80 &BYVAR &PV_ROOT.1-&PV_ROOT&NB_PV &EXPLICA &ID_SCHOOL);
	BY &BYVAR;
RUN;

%DO ITER_1 = 0 %TO 80;
	%DO ITER_2=1 %TO &NB_PV;
		PROC REG DATA=BRRDATA OUTEST=COEF_&ITER_2 NOPRINT EDF;
			MODEL&ITER_2: MODEL &PV_ROOT&ITER_2=&EXPLICA;
			WEIGHT &REPLI_ROOT&ITER_1;
			BY &BYVAR;
		RUN;
		PROC SORT DATA=COEF_&ITER_2; BY &BYVAR _DEPVAR_; run;
	%END;
	DATA COEF_TEMP; MERGE COEF_1-COEF_&NB_PV; BY &BYVAR _DEPVAR_; RUN; 

	DATA COEF_TEMP;
		SET COEF_TEMP;
		L=&ITER_1;
	RUN;

 	PROC APPEND BASE = BRR_TEMP1 DATA=COEF_TEMP;
 	RUN;

%END;

PROC SORT DATA=BRR_TEMP1;
	BY &BYVAR L;
RUN;

PROC TRANSPOSE DATA=BRR_TEMP1 OUT=BRR_TEMP PREFIX=PV;
	BY &BYVAR L;
	VAR INTERCEPT &EXPLICA _RSQ_;
RUN; 

DATA BRR_TEMP1 ;
	SET BRR_TEMP (RENAME=(_NAME_=CLASS));
	DROP _LABEL_;
RUN;

%LET I=1;
%DO %WHILE(%LENGTH(%SCAN(&EXPLICA,&I)));
	%LET I=%EVAL(&I+1);
%END;
%LET NB=%EVAL(&I-1);

%DO ITER_3=0 %TO &NB;
	%IF &ITER_3=0 %THEN %DO;
        %LET INDEP=Intercept;
	%END;
	%IF &ITER_3>0 %THEN %DO;
		%LET INDEP=%SCAN(&EXPLICA,&ITER_3);
	%END;

   	DATA BRR_TEMP1;
			SET BRR_TEMP1;
			IF (UPCASE(CLASS)=UPCASE("&INDEP")) THEN ORDRE=&ITER_3;
		    IF (SUBSTR(CLASS,1,5)="_RSQ_") THEN ORDRE=&I;
	RUN;
%END;

DATA BRR_TEMP2(KEEP=&BYVAR CLASS ORDRE PV1-PV&NB_PV)	BRR_TEMP3(KEEP=&BYVAR CLASS ORDRE FIN1-FIN&NB_PV STAT MESVAR FLAG_ST);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =SUM (OF PV1-PV&NB_PV)/&NB_PV;
		MESVAR=VAR (OF PV1-PV&NB_PV);
		ARRAY VECTR_1 (&NB_PV) PV1-PV&NB_PV; 
		ARRAY VECTR_2 (&NB_PV) FIN1-FIN&NB_PV; 
		DO BOUCLE_1=1 TO &NB_PV; 
	   		VECTR_2(BOUCLE_1)=VECTR_1(BOUCLE_1);
	   	END;
		ARRAY VECTR_3 (&NB_PV) PV1-PV&NB_PV; 
		DO BOUCLE_2=1 TO &NB_PV; 
			IF VECTR_3(BOUCLE_2) IN (.,.M,.N,.I) THEN FLAG_ST=+1; 
		END;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &BYVAR CLASS;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &BYVAR CLASS;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &BYVAR CLASS;
	ARRAY VECTR_4 (&NB_PV) PV1-PV&NB_PV;
	ARRAY VECTR_5 (&NB_PV) FIN1-FIN&NB_PV;
	ARRAY VECTR_6 (&NB_PV) VAR1-VAR&NB_PV;
	DO BOUCLE_3=1 TO &NB_PV;
	VECTR_6(BOUCLE_3)=(1/20)*((VECTR_4(BOUCLE_3)- VECTR_5(BOUCLE_3))**2);
	END;
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VAR1-VAR&NB_PV;
	BY &BYVAR CLASS;
	OUTPUT OUT=BRR_TEMP5 SUM=SS1-SS&NB_PV;
RUN;

DATA BRR_TEMP6;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &BYVAR CLASS;
	SAMP=SUM (OF SS1-SS&NB_PV)/&NB_PV;
	FACT_IMP=1+(1/&NB_PV);
	FINVAR=(SAMP+(FACT_IMP*MESVAR));
	SESTAT=(FINVAR)**0.5;
	IF FLAG_ST>=1 THEN DO; STAT=.; SESTAT=.; END; 
	FORMAT STAT F10.2;
	FORMAT SESTAT F10.2;
	KEEP &BYVAR CLASS STAT SESTAT ORDRE; 
RUN;

PROC SORT DATA=BRR_TEMP6 OUT=&OUTFILE (DROP=ORDRE);
	BY &BYVAR ORDRE;
RUN;


%IF (%UPCASE(&LIMIT)=YES) %THEN %DO;

	%LET NB_M=%EVAL(&NB+&NB_PV);

	DATA BRR_TEMP7;
		SET BRRDATA;
		NB_MISS=0;
		ARRAY VECTR_7 (&NB_M) &EXPLICA &PV_ROOT.1-&PV_ROOT&NB_PV;
		DO BOUCLE_4=1 TO &NB;
			IF (VECTR_7(BOUCLE_4) IN (.,.I,.M,.N)) THEN NB_MISS=NB_MISS+1;
		END;
		IF (NB_MISS>1) THEN NB_MISS=1;
	RUN;
	PROC FREQ DATA=BRR_TEMP7 NOPRINT;
		TABLE NB_MISS /OUT=BRR_TEMP8;
		BY &BYVAR;
		WHERE (NB_MISS=0);
	RUN;
	%LET FLAG_STUD=%SCAN(&LIMIT_CRITERIA,1);
	DATA BRR_TEMP8;
		SET BRR_TEMP8;
		IF (COUNT < &FLAG_STUD) THEN FLAG_STUD=1;
		ELSE FLAG_STUD=0;
		KEEP &BYVAR FLAG_STUD;
	RUN;
	PROC SORT DATA=BRR_TEMP7;
		BY &BYVAR &ID_SCHOOL;
	RUN;
	PROC FREQ DATA=BRR_TEMP7 NOPRINT;
		TABLE NB_MISS /OUT=BRR_TEMP9;
		BY &BYVAR &ID_SCHOOL;
		WHERE (NB_MISS=0);
	RUN;		
	PROC FREQ DATA=BRR_TEMP9 NOPRINT;
		TABLE NB_MISS /OUT=BRR_TEMP10;
		BY &BYVAR;
	RUN;
	%LET FLAG_SCH=%SCAN(&LIMIT_CRITERIA,2);
	DATA BRR_TEMP10;
		SET BRR_TEMP10;
		IF (COUNT < &FLAG_SCH) THEN FLAG_SCH=1;
		ELSE FLAG_SCH=0;
		KEEP &BYVAR FLAG_SCH;
	RUN;
	PROC SORT DATA=BRR_TEMP7;
		BY &BYVAR NB_MISS;
	RUN;
	PROC FREQ DATA=BRR_TEMP7 NOPRINT;
		TABLE NB_MISS/OUT=BRR_TEMP11;
		BY &BYVAR;
		WEIGHT &REPLI_ROOT.0;
	RUN;
	%LET K=1;
	%LET POPREF=;
	%LET NBVAR=%SCAN(&LIMIT_CRITERIA,4);
	%DO %WHILE(&K <= &NBVAR);
		%LET POPREF_ADD=%SCAN(&BYVAR,&K);
		%LET POPREF=&POPREF &POPREF_ADD;
		%LET K=%EVAL(&K+1);
	%END;
	
	PROC MEANS DATA=BRR_TEMP11 NOPRINT;
		VAR COUNT;
		BY &POPREF;
		OUTPUT OUT=BRR_TEMP12 SUM=SOMWGT;
	RUN;
	%LET FLAG_PCT=%SCAN(&LIMIT_CRITERIA,3);
	DATA BRR_TEMP13;
		MERGE BRR_TEMP11 BRR_TEMP12;
		BY &POPREF;
		PCT=(COUNT/SOMWGT)*100;
		IF (PCT < &FLAG_PCT) THEN FLAG_PCT=1;
		ELSE FLAG_PCT=0;
		IF (NB_MISS=0);
		KEEP &BYVAR FLAG_PCT;
	RUN;
	DATA &OUTFILE._CRITERIA;
		MERGE BRR_TEMP8 BRR_TEMP10 BRR_TEMP13;
		BY &BYVAR;
	RUN;
	PROC DATASETS LIBRARY=WORK NOLIST;
		DELETE BRR_TEMP7 BRR_TEMP8 BRR_TEMP9 BRR_TEMP10 BRR_TEMP11 BRR_TEMP12 BRR_TEMP13;
	RUN;
%END;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 BRR_TEMP6 COEF_TEMP COEF_1-COEF_&NB_PV BRRDATA;
RUN;

OPTIONS NOTES;

%MEND BRR_REG_nPV;
 
