%MACRO QUARTILE_nPV_NEW(INFILE=,
				   REPLI_ROOT=,
                   BYVAR = ,
                   PV_ROOT =,
				   NB_PV=,
                   INDEX =,
			   	   LIMIT=,
			   	   LIMIT_CRITERIA=,
			   	   ID_SCHOOL=,
                   OUTFILE =);

/* 

MEANING OF THE MACRO ARGUMENTS

INFILE			= 	INPUT DATA FILE.
REPLI_ROOT		= 	ROOT OF THE FINAL WEIGHT AND 80 REPLICATES VARIABLE NAMES. 
					FINAL WEIGHT VARIABLE NAME MUST BE THE REPLICATION ROOT FOLLOWED BY 0.
BYVAR			= 	BREAKDOWN VARIABLES
PV_ROOT			= 	ROOT OF THE PLAUSIBLE VALUES VARIABLES NAMES
NB_PV			=	NUMBER OF PLAUSIBLE VALUES
INDEX			=   VARIABLE NAME USED TO CREATE THE FOUR QUARTERS
LIMIT          	=    FLAGGING  YES OR NO
LIMIT_CRITERIA 	=   1) NUMBER OF STUDENTS 2) NUMBER OF SCHOOLS 3) PERCENTAGE OF STUDENTS AND 4) NUMBER OF VARIABLES FROM 
					THE BYVAR ARGUMENT FOR DEFINING THE POPULATION OF REFERENCE 
ID_SCHOOL	   	=	VARIABLE NAME FOR THE SCHOOL IDENTIFICATION

OUTFILE			= 	FILE WITH THE STATISTIC ESTIMATES AND THEIR STANDARD ERROR ESTIMATE

*/ 


OPTIONS NONOTES;

DATA 
	QUARTILE_TEMP1 (KEEP=&BYVAR &REPLI_ROOT.0-&REPLI_ROOT.80 &PV_ROOT.1-&PV_ROOT&NB_PV INDEX1-INDEX&NB_PV &ID_SCHOOL &INDEX NB_MISS) 
	QUARTILE_TEMP2 (KEEP=&BYVAR &REPLI_ROOT.0-&REPLI_ROOT.80 &PV_ROOT.1-&PV_ROOT&NB_PV INDEX1-INDEX&NB_PV &ID_SCHOOL &INDEX NB_MISS) ;	
	SET &INFILE;
	NB_MISS=0;
	ARRAY VECTR_1 (*) &INDEX &PV_ROOT.1-&PV_ROOT&NB_PV;
	DO BOUCLE_1=1 TO DIM(VECTR_1);
		IF (VECTR_1(BOUCLE_1) IN (.,.M,.N,.I)) THEN NB_MISS=NB_MISS+1;
	END;
	ARRAY VECTR_2 (&NB_PV) INDEX1-INDEX&NB_PV;
	DO BOUCLE_2= 1 TO &NB_PV; 
		KK=BOUCLE_2+1;
		NORSEED=-(BOUCLE_2*10+KK);
		VECTR_2(BOUCLE_2)=&INDEX + (0.0001*NORMAL(NORSEED));
	END; 
	IF (NB_MISS=0) THEN OUTPUT QUARTILE_TEMP1;
	OUTPUT QUARTILE_TEMP2; 
RUN;

%DO QUARTILE=1 %TO &NB_PV;

%SUBQUARTILEn(REPLICATE=&REPLI_ROOT,
                    Q_BYVAR=&BYVAR,
					Q_VAR=INDEX&QUARTILE,
                    VAR =&INDEX,
					PLAUSIBLE=&PV_ROOT&QUARTILE);
RUN;

DATA INDEX&QUARTILE;
	SET INDEX;
	STAT&QUARTILE=STAT;
	SESTAT&QUARTILE=SESTAT;
	KEEP &BYVAR CAT STAT&QUARTILE SESTAT&QUARTILE ;
RUN;

DATA RESULT&QUARTILE;
	SET RESULT;
	STAT&QUARTILE=STAT;
	SESTAT&QUARTILE=SESTAT;
	KEEP &BYVAR CAT STAT&QUARTILE SESTAT&QUARTILE ;
RUN;

%END;

DATA QUARTILE_TEMP5;
	MERGE INDEX1-INDEX&NB_PV;
	BY &BYVAR CAT;
	STAT= SUM (OF STAT1-STAT&NB_PV)/&NB_PV;
	VAR_IMP=VAR (OF STAT1-STAT&NB_PV);
	ARRAY VECTR_3 (&NB_PV) STAT1-STAT&NB_PV; 
	ARRAY VECTR_4 (&NB_PV) FIN1-FIN&NB_PV;
	DO BOUCLE_3=1 TO &NB_PV; 
		VECTR_4(BOUCLE_3)=VECTR_3(BOUCLE_3);
	END;

	ARRAY VECTR_5 (&NB_PV) SESTAT1-SESTAT&NB_PV;
	ARRAY VECTR_6 (&NB_PV) SE2_1 - SE2_&NB_PV;
	DO BOUCLE_4=1 TO &NB_PV; 
		VECTR_6(BOUCLE_4)=VECTR_5(BOUCLE_4)**2; 
	end; 
	VAR_SAMP=SUM ( OF SE2_1 - SE2_&NB_PV)/&NB_PV;
	FACT_IMP=1+(1/&NB_PV);
	FINVAR=(VAR_SAMP+(FACT_IMP*VAR_IMP));
	SESTAT=(FINVAR)**0.5;

	ARRAY VECTR_7 (&NB_PV) STAT1-STAT&NB_PV; 
	DO BOUCLE_5=1 TO &NB_PV; 
		IF VECTR_7(BOUCLE_5) IN (.,.M,.N,.I) THEN FLAG_ST=+1; 
	END;
	IF FLAG_ST>=1 THEN DO; STAT=.; SESTAT=.; END; 

	KEEP &BYVAR CAT STAT SESTAT;
RUN;

DATA QUARTILE_TEMP6;
	MERGE RESULT1-RESULT&NB_PV;
	BY &BYVAR CAT;
	STAT= SUM (OF STAT1-STAT&NB_PV)/&NB_PV;
	VAR_IMP=VAR (OF STAT1-STAT&NB_PV);
	ARRAY VECTR_8 (&NB_PV) STAT1-STAT&NB_PV; 
	ARRAY VECTR_9 (&NB_PV) FIN1-FIN&NB_PV;
	DO BOUCLE_6=1 TO &NB_PV; 
		VECTR_9(BOUCLE_6)=VECTR_8(BOUCLE_6);
	END;

	ARRAY VECTR_10 (&NB_PV) SESTAT1-SESTAT&NB_PV;
	ARRAY VECTR_11 (&NB_PV) SE2_1 - SE2_&NB_PV;
	DO BOUCLE_7=1 TO &NB_PV; 
		VECTR_11(BOUCLE_7)=VECTR_10(BOUCLE_7)**2; 
	END; 
	VAR_SAMP=SUM ( OF SE2_1 - SE2_&NB_PV)/&NB_PV;
	FACT_IMP=1+(1/&NB_PV);
	FINVAR=(VAR_SAMP+(FACT_IMP*VAR_IMP));
	SESTAT=(FINVAR)**0.5;

	ARRAY VECTR_12 (&NB_PV) STAT1-STAT&NB_PV; 
	DO BOUCLE_8=1 TO &NB_PV; 
		IF VECTR_12(BOUCLE_8) IN (.,.M,.N,.I) THEN FLAG_ST=+1; 
	END;
	IF FLAG_ST>=1 THEN DO; STAT=.; SESTAT=.; END; 

	KEEP &BYVAR CAT STAT SESTAT;
RUN;

DATA QUARTILE_TEMP7;
	MERGE 	QUARTILE_TEMP5 (RENAME=(STAT=INDEX_STAT SESTAT=INDEX_SESTAT))
			QUARTILE_TEMP6 (RENAME=(STAT=PV_STAT SESTAT=PV_SESTAT))	;
	BY &BYVAR CAT;
RUN;


%IF (%UPCASE(&LIMIT)=NO) %THEN %DO;

	DATA &OUTFILE;
		SET QUARTILE_TEMP7;
	RUN;

%END;
%ELSE %DO;
	PROC SORT  DATA=QUARTILE_TEMP1;
		BY &BYVAR;
	RUN;

	PROC MEANS DATA=QUARTILE_TEMP1 NOPRINT;
		VAR INDEX1-INDEX&NB_PV;
		BY &BYVAR;
		WEIGHT &REPLI_ROOT.0;
		WHERE (NB_MISS=0);
		OUTPUT OUT=QUARTILE_TEMP2 P25=Q1_1-Q1_&NB_PV P50=Q2_1-Q2_&NB_PV P75=Q3_1-Q3_&NB_PV;
	RUN;

	DATA 
		QUARTILE_TEMP3 (KEEP = &BYVAR &REPLI_ROOT.0-&REPLI_ROOT.80 &PV_ROOT.1-&PV_ROOT&NB_PV 
								INDEX1-INDEX&NB_PV CAT1-CAT&NB_PV &ID_SCHOOL &INDEX NB_MISS)
		QUARTILE_TEMP4 (KEEP = &BYVAR CAT1-CAT&NB_PV &REPLI_ROOT.0 NB_MISS &ID_SCHOOL &INDEX);
		MERGE QUARTILE_TEMP1 QUARTILE_TEMP2;
		BY &BYVAR;
		ARRAY VECTR_13 (&NB_PV) INDEX1-INDEX&NB_PV;
		ARRAY VECTR_14 (&NB_PV) Q1_1-Q1_&NB_PV;
		ARRAY VECTR_15 (&NB_PV) Q2_1-Q2_&NB_PV;
		ARRAY VECTR_16 (&NB_PV) Q3_1-Q3_&NB_PV;
		ARRAY VECTR_17 (&NB_PV) CAT1-CAT&NB_PV;
		DO BOUCLE_9=1 TO &NB_PV;
			IF (VECTR_13(BOUCLE_9) <= VECTR_14(BOUCLE_9)) THEN VECTR_17(BOUCLE_9)=1;
			IF (VECTR_13(BOUCLE_9) >  VECTR_14(BOUCLE_9) AND VECTR_13(BOUCLE_9) <= VECTR_15(BOUCLE_9)) THEN VECTR_17(BOUCLE_9)=2;
			IF (VECTR_13(BOUCLE_9) >  VECTR_15(BOUCLE_9) AND VECTR_13(BOUCLE_9) <= VECTR_16(BOUCLE_9)) THEN VECTR_17(BOUCLE_9)=3;
			IF (VECTR_13(BOUCLE_9) >  VECTR_16(BOUCLE_9)) THEN VECTR_17(BOUCLE_9)=4;
			IF (VECTR_13(BOUCLE_9) IN (.,.M,.N,.I)) THEN VECTR_17(BOUCLE_9) = .;
		END;
		IF (NB_MISS=0) THEN OUTPUT QUARTILE_TEMP3;
		OUTPUT QUARTILE_TEMP4; 
	RUN;
	%DO ITER_1=1 %TO &NB_PV; 
	PROC FREQ DATA=QUARTILE_TEMP3 NOPRINT;
		TABLE &BYVAR * CAT&ITER_1 /OUT=QUARTILE_TEMP8_&ITER_1;
	RUN;
	DATA QUARTILE_TEMP8_&ITER_1; SET QUARTILE_TEMP8_&ITER_1 (RENAME=(COUNT=C&ITER_1 CAT&ITER_1=CAT)); RUN; 
	%END; 
	%LET FLAG_STUD=%SCAN(&LIMIT_CRITERIA,1);
	DATA QUARTILE_TEMP8;
		MERGE QUARTILE_TEMP8_1 - QUARTILE_TEMP8_&NB_PV;
		BY &BYVAR ;
		C= SUM (OF C1-C&NB_PV)/&NB_PV;
		IF (C < &FLAG_STUD) THEN FLAG_STUD=1;
		ELSE FLAG_STUD=0;
		KEEP &BYVAR FLAG_STUD;
	RUN;

	%DO ITER_2=1 %TO &NB_PV;
	PROC FREQ DATA=QUARTILE_TEMP3 NOPRINT;
		TABLE &BYVAR * CAT&ITER_2 * &ID_SCHOOL /OUT=QUARTILE_TEMP9_&ITER_2;
	RUN;		
	PROC FREQ DATA=QUARTILE_TEMP9_&ITER_2 NOPRINT;
		TABLE &BYVAR * CAT&ITER_2 /OUT=QUARTILE_TEMP10_&ITER_2;
	RUN;
	DATA QUARTILE_TEMP10_&ITER_2; SET QUARTILE_TEMP10_&ITER_2 (RENAME=(COUNT=C&ITER_2 CAT&ITER_2=CAT)); RUN; 
	%END;
	%LET FLAG_SCH=%SCAN(&LIMIT_CRITERIA,2);
	DATA QUARTILE_TEMP10;
		MERGE QUARTILE_TEMP10_1 - QUARTILE_TEMP10_&NB_PV;
		BY &BYVAR CAT;
		C=SUM (OF C1- C&NB_PV)/&NB_PV;
		IF (C < &FLAG_SCH) THEN FLAG_SCH=1;
		ELSE FLAG_SCH=0;
		KEEP &BYVAR FLAG_SCH;
	RUN;

 	%LET K=1;
	%LET POPREF=;
	%LET NBVAR=%SCAN(&LIMIT_CRITERIA,4);
	%DO %WHILE(&K <= &NBVAR);
		%LET POPREF_ADD=%SCAN(&BYVAR,&K);
		%LET POPREF=&POPREF &POPREF_ADD;
		%LET K=%EVAL(&K+1);
	%END;

	%LET FLAG_PCT=%SCAN(&LIMIT_CRITERIA,3);
	
	%DO ITER_3=1 %TO &NB_PV;
	PROC FREQ DATA=QUARTILE_TEMP4 NOPRINT;
		TABLE &BYVAR * CAT&ITER_3 * NB_MISS / OUT=QUARTILE_TEMP11_&ITER_3;
		WEIGHT W_FSTR0;
	RUN;
	DATA QUARTILE_TEMP11_&ITER_3; SET QUARTILE_TEMP11_&ITER_3 (RENAME=(COUNT=C&ITER_3 CAT&ITER_3=CAT)); RUN; 
	%END;
	DATA QUARTILE_TEMP11;
		MERGE QUARTILE_TEMP11_1 - QUARTILE_TEMP11_&NB_PV;
		BY &BYVAR CAT NB_MISS;
	RUN;	

	PROC MEANS DATA=QUARTILE_TEMP11 NOPRINT;
		VAR C1-C&NB_PV;
		BY &BYVAR;
  		OUTPUT OUT=QUARTILE_TEMP12 SUM=SOMWGT1-SOMWGT&NB_PV;
	RUN;

	DATA QUARTILE_TEMP13;
		MERGE QUARTILE_TEMP11 QUARTILE_TEMP12;
		BY &BYVAR;
		IF (NB_MISS=0);
		ARRAY VECTR_18 (&NB_PV) C1-C&NB_PV;
		ARRAY VECTR_19 (&NB_PV) SOMWGT1-SOMWGT&NB_PV;
		ARRAY VECTR_20 (&NB_PV) PCT1-PCT&NB_PV;
		DO BOUCLE_10=1 TO &NB_PV;
			VECTR_20(BOUCLE_10)=(VECTR_18(BOUCLE_10)/VECTR_19(BOUCLE_10))*100;
		END;
		PCT=SUM( OF PCT1-PCT&NB_PV)/&NB_PV;
		IF (PCT < &FLAG_PCT) THEN FLAG_PCT=1;
		ELSE FLAG_PCT=0;
		KEEP &BYVAR FLAG_PCT;
	RUN;
	DATA &OUTFILE;
		MERGE QUARTILE_TEMP7 QUARTILE_TEMP8 QUARTILE_TEMP10 QUARTILE_TEMP13;
		BY &BYVAR;
	RUN;
	
	PROC DATASETS LIBRARY=WORK NOLIST;
		DELETE 
			QUARTILE_TEMP8_1 - QUARTILE_TEMP8_&NB_PV
			QUARTILE_TEMP8 
			QUARTILE_TEMP9_1 - QUARTILE_TEMP9_&NB_PV
			QUARTILE_TEMP10_1 - QUARTILE_TEMP10_&NB_PV
			QUARTILE_TEMP10
			QUARTILE_TEMP11_1 - QUARTILE_TEMP11_&NB_PV
			QUARTILE_TEMP11 QUARTILE_TEMP12 QUARTILE_TEMP13;
	RUN;

%END;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE  QUARTILE_TEMP1 QUARTILE_TEMP2 QUARTILE_TEMP3 QUARTILE_TEMP4 QUARTILE_TEMP5 QUARTILE_TEMP6 QUARTILE_TEMP7
			INDEX1 - INDEX&NB_PV INDEX
			RESULT1 - RESULT&NB_PV RESULT;
RUN;

OPTION NOTES;

%MEND ;



%MACRO SUBQUARTILEn(REPLICATE=,
                    Q_BYVAR=,
					Q_VAR= ,
                    VAR =,
					PLAUSIBLE=);

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP0;
RUN;

PROC SORT  DATA=QUARTILE_TEMP1;
	BY &Q_BYVAR;
RUN;

%DO ITER_4 = 0 %TO 80;

	PROC MEANS DATA=QUARTILE_TEMP1 NOPRINT;
		VAR &Q_VAR;
		BY &Q_BYVAR;
		WEIGHT &REPLI_ROOT&ITER_4;
		OUTPUT OUT=QUARTILE_TEMP3 P25=Q1 P50=Q2 P75=Q3;
	RUN;

	DATA 
		QUARTILE_TEMP4 (KEEP = &Q_BYVAR &REPLI_ROOT.0-&REPLI_ROOT.80 &PV_ROOT.1-&PV_ROOT&NB_PV 
								&Q_VAR CAT &ID_SCHOOL &INDEX &PLAUSIBLE) ;
		MERGE QUARTILE_TEMP1 QUARTILE_TEMP3;
		BY &Q_BYVAR;
		IF (&Q_VAR <= Q1) THEN CAT=1;
		IF (&Q_VAR >  Q1 AND &Q_VAR <= Q2) THEN CAT=2;
		IF (&Q_VAR >  Q2 AND &Q_VAR <= Q3) THEN CAT=3;
		IF (&Q_VAR >  Q3 ) THEN CAT=4;
	RUN;

	PROC SORT DATA=QUARTILE_TEMP4 OUT=BRRDATA;
  		BY &Q_BYVAR CAT;
	RUN;
 	PROC MEANS DATA=BRRDATA VARDEF=WGT NOPRINT;
		VAR &Q_VAR &PLAUSIBLE;
		BY &Q_BYVAR CAT;
		WEIGHT &REPLI_ROOT&ITER_4;
		OUTPUT OUT=MEAN_TEMP MEAN=PV1 PV2;
	RUN;

	DATA MEAN_TEMP;
		SET MEAN_TEMP;
		L=&ITER_4;
	RUN;

	PROC APPEND BASE = BRR_TEMP0 DATA=MEAN_TEMP;
 	RUN;
%END;

%DO VALUE=1 %TO 2;

DATA BRR_TEMP1;
	SET BRR_TEMP0 (RENAME=(PV&VALUE=PV));
RUN;
	
DATA BRR_TEMP2(KEEP=&Q_BYVAR CAT PV)BRR_TEMP3(KEEP=&Q_BYVAR CAT STAT);
	SET BRR_TEMP1;
	IF L > 0 THEN OUTPUT BRR_TEMP2;     
	ELSE DO;
		STAT =PV;
		OUTPUT BRR_TEMP3;
	END;
RUN;

PROC SORT DATA=BRR_TEMP2;
	BY &Q_BYVAR CAT;
RUN;

PROC SORT DATA=BRR_TEMP3;
	BY &Q_BYVAR CAT;
RUN;

DATA BRR_TEMP4;
	MERGE BRR_TEMP2 BRR_TEMP3;
	BY &Q_BYVAR CAT;
	VARI=((PV-STAT)**2)*(1/20);
RUN;

PROC UNIVARIATE DATA=BRR_TEMP4 NOPRINT;
	VAR VARI;
	BY &Q_BYVAR CAT;
	OUTPUT OUT=BRR_TEMP5 SUM=SS;
RUN;


DATA BRR_TEMP6;
	MERGE BRR_TEMP3 BRR_TEMP5;
	BY &Q_BYVAR CAT;
	SESTAT=(SS)**0.5;
	FORMAT STAT F10.2;
	FORMAT SESTAT F10.2;
	KEEP &Q_BYVAR CAT STAT SESTAT;
RUN;
%IF (&VALUE=1) %THEN %DO;
	DATA INDEX;
		SET BRR_TEMP6;
	RUN;
%END;
%IF (&VALUE=2) %THEN %DO;
	DATA RESULT;
		SET BRR_TEMP6;
	RUN;
%END;

PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE BRR_TEMP1 BRR_TEMP2 BRR_TEMP3 BRR_TEMP4 BRR_TEMP5 BRR_TEMP6 MEAN_TEMP BRRDATA;
RUN;

%END;
PROC DATASETS LIBRARY=WORK NOLIST;
	DELETE brr_temp0;
RUN;
%MEND SUBQUARTILEn;
